#ifndef _CONFIG_H_
#define _CONFIG_H_

#define DEBUG	1

#include <stdint.h>

/** ******************************************
*	@brief	CPU frequenzy
**********************************************/
#ifndef F_CPU
#error please define F_CPU
#endif

/** ******************************************
*	@brief	System informations
**********************************************/
#define VERSION		"V. 1.0.0beta"

#if defined (__AVR_ATmega32__)
#define CPU		"ATMEGA32"

/** ******************************************
*	@brief	Graphic Display Pins
**********************************************/
#define GLCD_WR_PIN		4
#define GLCD_WR_PORT	PORTB

#define GLCD_RD_PIN		3
#define GLCD_RD_PORT	PORTB

#define GLCD_CE_PIN		2
#define GLCD_CE_PORT	PORTB

#define GLCD_CD_PIN		1
#define GLCD_CD_PORT	PORTB

#define GLCD_RST_PIN	0
#define GLCD_RST_PORT	PORTB

#define GLCD_FS_PIN		6
#define GLCD_FS_PORT	PORTD

#define GLCD_LIGHT_PIN  7
#define GLCD_LIGHT_DDR  DDRD

#define GLCD_DATA_PORT 	PORTA
#define GLCD_DATA_PIN	PINA
#define GLCD_DATADIR	DDRA

// limits of (x,y) LCD graphics drawing
#define GLCD_XMAX 	240
#define GLCD_XMIN 	0
#define GLCD_YMAX 	128
#define GLCD_YMIN 	0

// Size of the display memory
#define GLCD_VRAM_SIZE		8192

/** ******************************************
*	@brief	Key Input Settings
**********************************************/
#define KEY_DDR         DDRC
#define KEY_PORT        PORTC
#define KEY_PIN         PINC
#define KEY_ENTER       0
#define KEY_MENU        1 // dummy
#define KEY_DOWN        1 // dummy
#define KEY_UP			1 // dummy
#define ALL_KEYS        (1<<KEY_ENTER | 1<<KEY_MENU | 1<<KEY_UP | 1<<KEY_DOWN)

/** ******************************************
*	@brief	Rotary switch
**********************************************/
#define PHASE_A_B_PUE   do { DDRC &= ~((1<<6) | (1<<7));  PORTC |= ((1<<6) | (1<<7)); } while(0)
#define PHASE_A     (PINC & 1<<6)
#define PHASE_B     (PINC & 1<<7)

/** ******************************************
*	@brief	GUI variables
**********************************************/
#define MAX_GUI_OBJECTS		20
#define MAX_GUI_LABEL_SIZE	20
#define MAX_GUI_TEXT_SIZE	40
#define GUI_MAX_TIMER		10

/** ******************************************
*	@brief	End of user settings
**********************************************/
#define SYSCLOCK	F_CPU
#define XTAL        F_CPU

#define	FALSE	0
#define	TRUE	(!FALSE)

#define YES	1
#define NO	0

#define HIGH	1
#define LOW	0

#define ON	1
#define OFF	0

#define ENABLE	1
#define DISABLE	0

#define SOLID		0b1
#define TRANS		0b10
#define INVERSE		0b100
#define FRAME		0b1000

#else
#define CPU		"ATMEGA 328p"

/** ******************************************
*	@brief	Graphic Display Pins
**********************************************/
#define GLCD_WR_PIN		2
#define GLCD_WR_PORT	PORTD

#define GLCD_RD_PIN		3
#define GLCD_RD_PORT	PORTD

#define GLCD_CE_PIN		4
#define GLCD_CE_PORT	PORTD

#define GLCD_CD_PIN		5
#define GLCD_CD_PORT	PORTD

#define GLCD_RST_PIN	6
#define GLCD_RST_PORT	PORTD

#define GLCD_FS_PIN		7
#define GLCD_FS_PORT	PORTD

#define GLCD_DATA_PORT 	PORTB
#define GLCD_DATA_PIN	PINB
#define GLCD_DATADIR	DDRB

// limits of (x,y) LCD graphics drawing
#define GLCD_XMAX 240
#define GLCD_XMIN 0
#define GLCD_YMAX 128
#define GLCD_YMIN 0

// Size of the display memory
#define GLCD_VRAM_SIZE		8192

/** ******************************************
*	@brief	Key Input Settings
**********************************************/
#define KEY_DDR         DDRC
#define KEY_PORT        PORTC
#define KEY_PIN         PINC
#define KEY_ENTER       0
#define KEY_MENU        1
#define KEY_DOWN        2
#define KEY_UP			3
#define ALL_KEYS        (1<<KEY_ENTER | 1<<KEY_MENU | 1<<KEY_UP | 1<<KEY_DOWN)

/** ******************************************
*	@brief	Rotary switch
**********************************************/
#define PHASE_A     (PIND & 1<<PD0)
#define PHASE_B     (PIND & 1<<PD1)

/** ******************************************
*	@brief	GUI variables
**********************************************/
#define MAX_GUI_OBJECTS		20
#define MAX_GUI_LABEL_SIZE	20
#define MAX_GUI_TEXT_SIZE	40
#define GUI_MAX_TIMER		10

/** ******************************************
*	@brief	End of user settings
**********************************************/
#define SYSCLOCK	F_CPU
#define XTAL        F_CPU

#define	TRUE	1
#define	FALSE	0

#define YES	1
#define NO	0

#define HIGH	1
#define LOW	0

#define ON	1
#define OFF	0

#define ENABLE	1
#define DISABLE	0

#define SOLID		0b1
#define TRANS		0b10
#define INVERSE		0b100
#define FRAME		0b1000
#endif
/** ******************************************
*	@brief	Port corrections
**********************************************/
#if defined(__AVR_ATmega328P__)
  #define PB0	PINB0
  #define PB1	PINB1
  #define PB2	PINB2
  #define PB3	PINB3
  #define PB4	PINB4
  #define PB5	PINB5
  #define PB6	PINB6
  #define PB7	PINB7

  #define PC0	PINC0
  #define PC1	PINC1
  #define PC2	PINC2
  #define PC3	PINC3
  #define PC4	PINC4
  #define PC5	PINC5
  #define PC6	PINC6

  #define PD0	PIND0
  #define PD1	PIND1
  #define PD2	PIND2
  #define PD3	PIND3
  #define PD4	PIND4
  #define PD5	PIND5
  #define PD6 	PIND6
  #define PD7	PIND7

#endif

/** ******************************************
*	@brief	Timer stuff
**********************************************/
extern volatile uint8_t f_timer_tick;

#define TIMERTICK	(F_CPU / 256 / 256)	// timer ticks per second


extern char ram_list[3][MAX_GUI_LABEL_SIZE];
extern volatile int MESSAGE_COUNTER;

#endif /* _CONFIG_H_ */
