#ifndef _T6963C_H_
#define _T6963C_H_

/*
 *  FG                (1)  frame ground
 *  GND               (2)  ground
 *  VDD               (3)  LCD logic supply  +5 DC
 *  VO                (4)  Contrast
 *  /WR               (5)  <-->
 *  /RD               (6)  <-->
 *  /CE or /CS        (7)  <-->
 *  CD or RS          (8)  <-->
 *  /RST              (9)  <-->

 *  D0                (10) <-->
 *  D1                (11) <-->
 *  D2                (12) <-->
 *  D3                (13) <-->
 *  D4                (14) <-->
 *  D5                (15) <-->
 *  D6                (16) <-->
 *  D7                (17) <-->

 *  FS                (18)           font select       HI : narrow            LOW : Wide   ( normal )
                                                       40 with 6x8 font       30 with 8x8 font
 *  Vout              (19)           Variable Resistor (~5K Ohm) with GND and centre with VO ( Pin 4 ). Used for contrast adjustment.
 *  BLA               (20)           Back light + (+5V DC)
 *  BLK               (21)           Back light - (GND)
 *  DSPON             (22)           I don't Know   but i think means display ON or ...
 */

#include <avr/io.h>
#include <stdint.h>
#include "config.h"

#define GLCD_WIDTH				GLCD_XMAX
#define GLCD_HEIGHT				GLCD_YMAX
//#define GLCD_ADDRESSES			128
#define GLCD_SIZE				(GLCD_WIDTH * GLCD_HEIGHT)

#if ( GLCD_WIDTH < 256)
  #define glcd_x_t	uint8_t
#elif  ( GLCD_WIDTH <= 0xFFFF )
  #define glcd_x_t	uint16_t
#else
  #define glcd_x_t	uint32_t
#endif

#if ( GLCD_HEIGHT < 256)
  #define glcd_y_t	uint8_t
#elif  ( GLCD_HEIGHT <= 0xFFFF )
  #define glcd_y_t	uint16_t
#else
  #define glcd_y_t	uint32_t
#endif

#define GLCD_FONT_WIDTH 		8				// pixel width
#define GLCD_BYTES_PER_ROW 		(GLCD_XMAX/GLCD_FONT_WIDTH)
#define GLCD_G_BASE			0x0000				// base address of graphics memory
#define GLCD_MAX_PAGES			(GLCD_VRAM_SIZE/(GLCD_G_BASE+GLCD_BYTES_PER_ROW*GLCD_YMAX))

#define GLCD_WR_HIGH() GLCD_WR_PORT  |=  _BV(GLCD_WR_PIN);
#define GLCD_WR_LOW()  GLCD_WR_PORT  &= ~_BV(GLCD_WR_PIN);

#define GLCD_RD_HIGH() GLCD_RD_PORT  |=  _BV(GLCD_RD_PIN);
#define GLCD_RD_LOW()  GLCD_RD_PORT  &= ~_BV(GLCD_RD_PIN);

#define GLCD_CE_HIGH() GLCD_CE_PORT  |=  _BV(GLCD_CE_PIN);
#define GLCD_CE_LOW()  GLCD_CE_PORT  &= ~_BV(GLCD_CE_PIN);

#define GLCD_CD_HIGH() GLCD_CD_PORT  |=  _BV(GLCD_CD_PIN);
#define GLCD_CD_LOW()  GLCD_CD_PORT  &= ~_BV(GLCD_CD_PIN);

#define GLCD_RST_HIGH() GLCD_RST_PORT  |=  _BV(GLCD_RST_PIN);
#define GLCD_RST_LOW()  GLCD_RST_PORT  &= ~_BV(GLCD_RST_PIN);

#define GLCD_FS_HIGH() GLCD_FS_PORT  |=  _BV(GLCD_FS_PIN);
#define GLCD_FS_LOW()  GLCD_FS_PORT  &= ~_BV(GLCD_FS_PIN);

#define GLCD_SET_DATADIR_OUT()	(GLCD_DATADIR = 0xFF)
#define GLCD_SET_DATADIR_IN()	(GLCD_DATADIR = 0x00)

#define DDR(x) (*(&x - 1))     //address of data direction register of port x
#if defined(__AVR_ATmega64__) || defined(__AVR_ATmega128__)
   //on ATmega64/128 PINF is on port 0x00 and not 0x60
    #define PIN(x) ( &PORTF==&(x) ? _SFR_IO8(0x00) : (*(&x - 2)) )
#else
	#define PIN(x) (*(&x - 2))    //address of input register of port x
#endif


//extern void GLCD_SET_ADDRESS(uint16_t addr);
extern void GLCD_CLEAR(void);
extern void GLCD_PIXEL(glcd_x_t x, glcd_y_t y, uint8_t show);
extern void GLCD_INIT(void);
//extern uint8_t GLCD_SGET(void);
extern void GLCD_DPUT(uint8_t byte);
extern uint8_t GLCD_DGET(void);
extern void GLCD_CPUT(uint8_t byte);
extern void GLCD_GOTO(glcd_x_t x, glcd_y_t y);
extern void GLCD_SET_PAGE(uint_fast8_t page);
extern void GLCD_CLEAR_PAGE(uint_fast8_t page);
extern void GLCD_COPY_PAGE(uint_fast8_t source, uint_fast8_t dest);
extern uint8_t GLCD_GETBYTE_noinc(void);
extern void GLCD_PUTBYTE_inc(uint8_t byte);
#endif



