#include <avr/pgmspace.h>
#include "include/config.h"
#include "include/font.h"

/** *************************************************************
	@brief	Print a String from flash
	@param 	x 	: column
	@param 	y 	: row
	@param 	in 	: The String
	@param 	fnt : Struct with the font
	@param 	invers : 0=Normal , 1= Invers, 2 = only set if unset, 3 = clear if set
	@return	none
*****************************************************************/

void GLCD_PRINT_P(glcd_x_t x, glcd_y_t y, const char *in, const struct FONT_DEF * const fnt, uint8_t invers)
{
	uint16_t offset;
	uint8_t width, by = 0, mask = 0;
	uint8_t NrBytes;
	uint8_t i, j, map, ertefah, allwidth = 0;
	uint8_t *table;

	while((map = pgm_read_byte(in++)) != '\0')
	{
		table = (uint8_t *)pgm_read_word(&fnt->mapping_table);
		map = pgm_read_byte(&table[map]);

		width = pgm_read_byte(&fnt->glyph_width);
		if(width == 0)
		{
			table = (uint8_t *)pgm_read_word(&fnt->width_table);
			width = pgm_read_byte(&table[map]);
        }
        table = (uint8_t *)pgm_read_word(&fnt->offset_table);
		offset = pgm_read_word(&((uint16_t *)table)[map]);
  		ertefah = pgm_read_byte(&fnt->glyph_height);		// Die Hoehe des Fonts

		NrBytes = ((width-1)/8)+1;

        table = (uint8_t *)pgm_read_word(&fnt->glyph_table);
		for(j = 0; j < ertefah * NrBytes; j += NrBytes )
		{   // ertefah
		    mask = 0;
			for(i = 0; i < width; i++)
			{   //  width
			    if(mask == 0)
			    {
			     	by = pgm_read_byte(&table[ offset+j+(i/8) ]);
			     	mask = 1 << 7;
			    }

                if(((invers < 2) || (by & mask)) && ((x+i+allwidth) < GLCD_XMAX) && ((y+j/ NrBytes) < GLCD_YMAX))
					GLCD_PIXEL( x+i+allwidth , y+j/ NrBytes, (by & mask) ? !(invers & 1) : (invers & 1)  );

	 			mask >>= 1;
			}//End i
		}// End j
		allwidth += width;
	}// End K
}

/** *************************************************************
	@brief	Print a String
	@param 	x 	: column
	@param 	y 	: row
	@param 	in 	: The String
	@param 	fnt : Struct with the font
	@param 	invers : 0=Normal , 1= Invers, 2 = only set if unset, 3 = clear if set
	@return	none
*****************************************************************/
void GLCD_PRINT(glcd_x_t x, glcd_y_t y, const char *in, const struct FONT_DEF * const fnt, uint8_t invers)
{
	unsigned int offset;
	uint8_t width, by = 0, mask = 0;
	uint8_t NrBytes;
	uint8_t i, j, map, allwidth = 0;

    uint8_t  *mapping_table = (uint8_t *)pgm_read_word(&fnt->mapping_table);
  	uint8_t	 ertefah = pgm_read_byte(&fnt->glyph_height);		// Die Hoehe des Fonts
	uint8_t	 *glyph_table = (uint8_t *)pgm_read_word(&fnt->glyph_table);
	uint16_t *offset_table = (uint16_t *)pgm_read_word(&fnt->offset_table);

	while((map = *in++))
	{
		map = pgm_read_byte(&mapping_table[map]);

		width = pgm_read_byte(&fnt->glyph_width);
		if(width == 0)
		{
			width = pgm_read_byte(pgm_read_word(&fnt->width_table) + map);
        }
		offset = pgm_read_word(&offset_table[map]);

		NrBytes = ((width-1)/8)+1;

		for(j = 0; j < ertefah * NrBytes; j += NrBytes)
		{   // ertefah
		    mask = 0;
			for(i = 0; i < width; i++)
			{   //  width
			    if(mask == 0)
			    {
			     by = pgm_read_byte(&glyph_table[ offset+j+(i/8) ]);
			     mask = 1 << 7;
			    }
                if(((invers < 2) || (by & mask)) && ((x+i+allwidth) < GLCD_XMAX) && ((y+j/ NrBytes) < GLCD_YMAX))
				    GLCD_PIXEL(  x+i+allwidth , y+j/NrBytes, (by & mask) ? !(invers & 1) : (invers & 1) );

	 			mask >>= 1;
			}//End i
		}// End j
		allwidth += width;
	}// End K
}

/** *************************************************************
	@brief	Get lenght in pixels for a string
	@param 	in 	: The String
	@param 	fnt : Struct with the font
	@return	none
*****************************************************************/
int GLCD_STRLEN(const char *in, const struct FONT_DEF * const fnt)
{
	uint8_t width;
	uint8_t map;
	int allwidth = 0;
	uint8_t *w_table, *m_table;

	w_table = (uint8_t *)pgm_read_word(&fnt->width_table);
	m_table = (uint8_t *)pgm_read_word(&fnt->mapping_table);
	width = pgm_read_byte(&fnt->glyph_width);
	while((map = *in++))
	{
		map = pgm_read_byte(&m_table[map]);

		if(width == 0)
			allwidth += pgm_read_byte(&w_table[map]);
        else
			allwidth += width;
	}// End K
	return allwidth;
}

/** *************************************************************
	@brief	Get lenght in pixels for a string in flash
	@param 	in 	: The String
	@param 	fnt : Struct with the font
	@return	none

*****************************************************************/
int GLCD_STRLEN_P(const char *in, const struct FONT_DEF * const fnt)
{
	uint8_t width;
	uint8_t map;
	int allwidth = 0;
	uint8_t *w_table, *m_table;

	w_table = (uint8_t *)pgm_read_word(&fnt->width_table);
	m_table = (uint8_t *)pgm_read_word(&fnt->mapping_table);
	width = pgm_read_byte(&fnt->glyph_width);
	while((map = pgm_read_byte(in++)))
	{
		map = pgm_read_byte(&m_table[map]);

		if(width == 0)
			allwidth += pgm_read_byte(&w_table[map]);
        else
			allwidth += width;
	}// End K
	return allwidth;
}

/** *************************************************************
	@brief	Print a character
	@param 	x 	: column
	@param 	y 	: row
	@param 	in 	: The String
	@param 	fnt : Struct with the font
	@return	width in pixel
*****************************************************************/
uint8_t GLCD_CHAR(glcd_x_t x, glcd_y_t y, char in, const struct FONT_DEF * const fnt)
{
  	uint8_t  map;
  	uint8_t  width;
  	uint16_t offset;
  	uint8_t  height;
  	uint8_t *table;

  	// in == Index des Zeichen innerhalb des Font
  	table = (uint8_t *)pgm_read_word(&fnt->mapping_table);
  	map   = pgm_read_byte(&table[(uint8_t)in]);		// Internal code

	// Die Hoehe des Font
  	height = pgm_read_byte(&fnt->glyph_height);

  	// Breite des Zeichen
  	width = pgm_read_byte(&fnt->glyph_width);
  	if(width == (uint8_t)0)	// Proportionaler Font
  	{
  		table = (uint8_t *)pgm_read_word(&fnt->width_table);
  		width = pgm_read_byte(&table[map]);	// Breite des Zeichens
	}

    // Die Adresse bei der das Zeichen anfaengt
 	table = (uint8_t *)pgm_read_word(&fnt->offset_table);
  	offset = pgm_read_word(&((uint16_t *)table)[map]);

  	// Font Tabelle
  	table = (uint8_t *)pgm_read_word(&fnt->glyph_table);

	GLCD_plot_bitmap(x, y, &table[offset], width, height, TRUE);

	return width;
}


/** *************************************************************
	@brief	Get the hight of a font
	@param none
	@return	The height
*****************************************************************/
int GLCD_GET_FONT_HEIGHT( const struct FONT_DEF * const fnt)
{
  return pgm_read_byte(&fnt->glyph_height);		// Die Hoehe des Fonts
}

/**
 * This function will plot a bitmap with the upper left corner to the given coordinates.\n
 * If the bitmap doesn't fit on the display at this location, nothing is drawn at all.\n
 * Unset pixels are treated as transparent \n
 * This function is by far the most tricky piece of code in this project,\n
 * don't worry if you don't understand it at first sight. \n
 * And if you know a better way to do this, tell me.\n
 *
 * @param x horizontal position of the bitmap
 * @param y vertical position of the bitmap
 * @param bitmap pointer to the bitmap in pgmspace
 * @param w width of the bitmap
 * @param h height of the bitmap
 * @param rev  TRUE if bitmap starts from bit 7; FALSE if bitmap starts from bit 0.
 *
 * This function is dedicated to Greta, one of the most important persons in my life so far.\n
 * Even though I can't remember her actual name.\n
 *
 * The concept of this part is taken from Sebastians AVR Library for lc7981 available at
 * http://sebastians-site.de/homepage/ rsp http://sebastians-site.de/hg_repos/glcd/
 */
#define REV  1      // Set to 1 if bitmap start from bit 7; Set to 0 if bitmap starts from bit 0
void GLCD_plot_bitmap(glcd_x_t x_off, glcd_y_t y_off, const uint8_t * bitmap, glcd_x_t w, glcd_y_t h, uint8_t rev)
{
	glcd_x_t x;
	glcd_y_t y;
	uint8_t cur = 0, curs = 0, sr, p;

	//check if the bitmap fits on the display
	if((x_off + w < GLCD_XMAX) && (y_off + h < GLCD_YMAX))
	{
		//loop linewise through the bitmap
		for(y = y_off; y < y_off + h; y++)
		{
			GLCD_GOTO(x_off, y);
			p = 0;
			//loop pixelwise through each line
			for(x = x_off & 0xF8; x_off + w > x; x++)
			{
				if(p == 0)   //load the next byte from display memory
				{
      				// Lese Display ohne adresse zu aendern
					cur = GLCD_GETBYTE_noinc();
				    p = rev ? 0x80 : 0x01;
				}
				if(x >= x_off)
				{
					sr = (x - x_off) & 7;
					if(sr == 0)   //load the next byte of the bitmap
					{
						curs = pgm_read_byte(bitmap++);
					}
					//grep the pixel from the bitmap and put into the display byte
					if (rev)
					{
 					  if(curs & 0x80)
						cur |= p; //(1 << dr);
					  /* Activate the else if you need overwrite mode */
					  //else cur &= ~p;
					  curs <<= 1;
					}
					else
					{
					  if(curs & 1)
						cur |= p; //(1 << dr);
					  /* Activate the else if you need overwrite mode */
					  //else cur &= ~p;
					  curs >>= 1;
					}
				}
				p = rev ? (p >> 1) : (p << 1);
				if(p == 0)   //write the current byte to display memory
				{
					GLCD_GOTO(x,y);
      				GLCD_PUTBYTE_inc(cur);	// Den buffer ausgeben und erhoehe die adresse
				}
			}
			if(p != 0)   //write the last byte to display memory if there's one left
			{
				GLCD_GOTO(x,y);
   				GLCD_PUTBYTE_inc(cur);	// Den buffer ausgeben und erhoehe die adresse
			}
		}
	}
}

